@@ -23,7 +23,7 @@ from xml.sax.saxutils import quoteattr, escape
 from sphinx.errors import ConfigError, ExtensionError
 from sphinx.util.console import red
 from . import sphinx_compatibility, glr_path_static, __version__ as _sg_version
-from .utils import _replace_md5, _has_optipng
+from .utils import _replace_md5, _has_optipng, _has_pypandoc
 from .backreferences import _finalize_backreferences
 from .gen_rst import (generate_dir_rst, SPHX_GLR_SIG, _get_memory_base,
                       _get_readme)
@@ -31,16 +31,27 @@ from .scrapers import _scraper_dict, _reset_dict, _import_matplotlib
 from .docs_resolv import embed_code_links
 from .downloads import generate_zipfiles
 from .sorting import NumberOfCodeLinesSortKey
-from .binder import copy_binder_files
+from .binder import copy_binder_files, check_binder_conf
 from .directives import MiniGallery
 
 
-_KNOWN_CSS = ('gallery', 'gallery-binder', 'gallery-dataframe')
+_KNOWN_CSS = ('gallery', 'gallery-binder', 'gallery-dataframe',
+              'gallery-rendered-html')
+
+
+class DefaultResetArgv:
+    def __repr__(self):
+        return "DefaultResetArgv"
+
+    def __call__(self, gallery_conf, script_vars):
+        return []
+
 
 DEFAULT_GALLERY_CONF = {
     'filename_pattern': re.escape(os.sep) + 'plot',
     'ignore_pattern': r'__init__\.py',
     'examples_dirs': os.path.join('..', 'examples'),
+    'reset_argv': DefaultResetArgv(),
     'subsection_order': None,
     'within_subsection_order': NumberOfCodeLinesSortKey,
     'gallery_dirs': 'auto_examples',
@@ -48,6 +59,7 @@ DEFAULT_GALLERY_CONF = {
     'doc_module': (),
     'reference_url': {},
     'capture_repr': ('_repr_html_', '__repr__'),
+    'ignore_repr_types': r'',
     # Build options
     # -------------
     # We use a string for 'plot_gallery' rather than simply the Python boolean
@@ -59,6 +71,7 @@ DEFAULT_GALLERY_CONF = {
     'failing_examples': {},
     'passing_examples': [],
     'stale_examples': [],  # ones that did not need to be run due to md5sum
+    'run_stale_examples': False,
     'expected_failing_examples': set(),
     'thumbnail_size': (400, 280),  # Default CSS does 0.4 scaling (160, 112)
     'min_reported_time': 0,
@@ -68,12 +81,14 @@ DEFAULT_GALLERY_CONF = {
     'reset_modules': ('matplotlib', 'seaborn'),
     'first_notebook_cell': '%matplotlib inline',
     'last_notebook_cell': None,
+    'notebook_images': False,
+    'pypandoc': False,
     'remove_config_comments': False,
     'show_memory': False,
+    'show_signature': True,
     'junit': '',
     'log_level': {'backreference_missing': 'warning'},
     'inspect_global_variables': True,
-    'ignore_repr_types': r'',
     'css': _KNOWN_CSS,
     'matplotlib_animations': False,
 }
@@ -81,14 +96,21 @@ DEFAULT_GALLERY_CONF = {
 logger = sphinx_compatibility.getLogger('sphinx-gallery')
 
 
+def _bool_eval(x):
+    if isinstance(x, str):
+        try:
+            x = eval(x)
+        except TypeError:
+            pass
+    return bool(x)
+
+
 def parse_config(app):
-    """Process the Sphinx Gallery configuration"""
-    try:
-        plot_gallery = eval(app.builder.config.plot_gallery)
-    except TypeError:
-        plot_gallery = bool(app.builder.config.plot_gallery)
+    """Process the Sphinx Gallery configuration."""
+    plot_gallery = _bool_eval(app.builder.config.plot_gallery)
     src_dir = app.builder.srcdir
-    abort_on_example_error = app.builder.config.abort_on_example_error
+    abort_on_example_error = _bool_eval(
+        app.builder.config.abort_on_example_error)
     lang = app.builder.config.highlight_language
     gallery_conf = _complete_gallery_conf(
         app.config.sphinx_gallery_conf, src_dir, plot_gallery,
@@ -114,6 +136,10 @@ def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
         gallery_conf['image_scrapers'] += ('mayavi',)
     gallery_conf.update(plot_gallery=plot_gallery)
     gallery_conf.update(abort_on_example_error=abort_on_example_error)
+    # XXX anything that can only be a bool (rather than str) should probably be
+    # evaluated this way as it allows setting via -D on the command line
+    for key in ('run_stale_examples',):
+        gallery_conf[key] = _bool_eval(gallery_conf[key])
     gallery_conf['src_dir'] = src_dir
     gallery_conf['app'] = app
 
@@ -131,6 +157,22 @@ def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
             backreferences_warning,
             type=DeprecationWarning)
 
+    # Check capture_repr
+    capture_repr = gallery_conf['capture_repr']
+    supported_reprs = ['__repr__', '__str__', '_repr_html_']
+    if isinstance(capture_repr, tuple):
+        for rep in capture_repr:
+            if rep not in supported_reprs:
+                raise ConfigError("All entries in 'capture_repr' must be one "
+                                  "of %s, got: %s" % (supported_reprs, rep))
+    else:
+        raise ConfigError("'capture_repr' must be a tuple, got: %s"
+                          % (type(capture_repr),))
+    # Check ignore_repr_types
+    if not isinstance(gallery_conf['ignore_repr_types'], str):
+        raise ConfigError("'ignore_repr_types' must be a string, got: %s"
+                          % (type(gallery_conf['ignore_repr_types']),))
+
     # deal with show_memory
     gallery_conf['memory_base'] = 0.
     if gallery_conf['show_memory']:
@@ -254,6 +296,31 @@ def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
     if (not isinstance(last_cell, str)) and (last_cell is not None):
         raise ConfigError("The 'last_notebook_cell' parameter must be type str"
                           " or None, found type %s" % type(last_cell))
+    # Check pypandoc
+    pypandoc = gallery_conf['pypandoc']
+    if not isinstance(pypandoc, (dict, bool)):
+        raise ConfigError("'pypandoc' parameter must be of type bool or dict,"
+                          "got: %s." % type(pypandoc))
+    gallery_conf['pypandoc'] = dict() if pypandoc is True else pypandoc
+    has_pypandoc, version = _has_pypandoc()
+    if isinstance(gallery_conf['pypandoc'], dict) and has_pypandoc is None:
+        logger.warning("'pypandoc' not available. Using Sphinx-Gallery to "
+                       "convert rst text blocks to markdown for .ipynb files.")
+        gallery_conf['pypandoc'] = False
+    elif isinstance(gallery_conf['pypandoc'], dict):
+        logger.info("Using pandoc version: %s to convert rst text blocks to "
+                    "markdown for .ipynb files" % (version,))
+    else:
+        logger.info("Using Sphinx-Gallery to convert rst text blocks to "
+                    "markdown for .ipynb files.")
+    if isinstance(pypandoc, dict):
+        accepted_keys = ('extra_args', 'filters')
+        for key in pypandoc:
+            if key not in accepted_keys:
+                raise ConfigError("'pypandoc' only accepts the following key "
+                                  "values: %s, got: %s."
+                                  % (accepted_keys, key))
+
     # Make it easy to know which builder we're in
     gallery_conf['builder_name'] = builder_name
     gallery_conf['titles'] = {}
@@ -269,6 +336,9 @@ def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
     if isinstance(backref, pathlib.Path):
         gallery_conf['backreferences_dir'] = str(backref)
 
+    # binder
+    gallery_conf['binder'] = check_binder_conf(gallery_conf['binder'])
+
     if not isinstance(gallery_conf['css'], (list, tuple)):
         raise ConfigError('gallery_conf["css"] must be list or tuple, got %r'
                           % (gallery_conf['css'],))
@@ -335,7 +405,7 @@ def _prepare_sphx_glr_dirs(gallery_conf, srcdir):
 def generate_gallery_rst(app):
     """Generate the Main examples gallery reStructuredText
 
-    Start the sphinx-gallery configuration and recursively scan the examples
+    Start the Sphinx-Gallery configuration and recursively scan the examples
     directories in order to populate the examples gallery
     """
     logger.info('generating gallery...', color='white')
@@ -384,11 +454,13 @@ def generate_gallery_rst(app):
                 write_computation_times(gallery_conf, target_dir, this_costs)
 
             if gallery_conf['download_all_examples']:
-                download_fhindex = generate_zipfiles(gallery_dir)
+                download_fhindex = generate_zipfiles(
+                    gallery_dir, app.builder.srcdir)
                 fhindex.write(download_fhindex)
 
-            fhindex.write(SPHX_GLR_SIG)
-        _replace_md5(index_rst_new)
+            if (app.config.sphinx_gallery_conf['show_signature']):
+                fhindex.write(SPHX_GLR_SIG)
+        _replace_md5(index_rst_new, mode='t')
     _finalize_backreferences(seen_backrefs, gallery_conf)
 
     if gallery_conf['plot_gallery']:
@@ -588,7 +660,7 @@ def summarize_failing_examples(app, exception):
 
     # Under no-plot Examples are not run so nothing to summarize
     if not app.config.sphinx_gallery_conf['plot_gallery']:
-        logger.info('Sphinx-gallery gallery_conf["plot_gallery"] was '
+        logger.info('Sphinx-Gallery gallery_conf["plot_gallery"] was '
                     'False, so no examples were executed.', color='brown')
         return
 
@@ -623,7 +695,7 @@ def summarize_failing_examples(app, exception):
     n_good = len(gallery_conf['passing_examples'])
     n_tot = len(gallery_conf['failing_examples']) + n_good
     n_stale = len(gallery_conf['stale_examples'])
-    logger.info('\nSphinx-gallery successfully executed %d out of %d '
+    logger.info('\nSphinx-Gallery successfully executed %d out of %d '
                 'file%s subselected by:\n\n'
                 '    gallery_conf["filename_pattern"] = %r\n'
                 '    gallery_conf["ignore_pattern"]   = %r\n'
@@ -694,7 +766,7 @@ def get_default_config_value(key):
 
 
 def setup(app):
-    """Setup sphinx-gallery sphinx extension"""
+    """Setup Sphinx-Gallery sphinx extension"""
     sphinx_compatibility._app = app
 
     app.add_config_value('sphinx_gallery_conf', DEFAULT_GALLERY_CONF, 'html')
