@@ -10,8 +10,7 @@ example files.
 
 Files that generate images should start with 'plot'.
 """
-# Don't use unicode_literals here (be explicit with u"..." instead) otherwise
-# tricky errors come up with exec(code_blocks, ...) calls
+
 from __future__ import division, print_function, absolute_import
 from time import time
 import copy
@@ -25,6 +24,7 @@ import importlib
 from io import StringIO
 import os
 import re
+import stat
 from textwrap import indent
 import warnings
 from shutil import copyfile
@@ -138,8 +138,7 @@ SINGLE_IMAGE = """
     :class: sphx-glr-single-img
 """
 
-# This one could contain unicode
-CODE_OUTPUT = u""".. rst-class:: sphx-glr-script-out
+CODE_OUTPUT = """.. rst-class:: sphx-glr-script-out
 
  Out:
 
@@ -163,22 +162,24 @@ SPHX_GLR_SIG = """\n
 """
 
 # Header used to include raw html
-html_header = """.. only:: builder_html
+html_header = """.. raw:: html
 
-    .. raw:: html
-
-{0}\n        <br />\n        <br />"""
+    <div class="output_subarea output_html rendered_html output_result">
+{0}
+    </div>
+    <br />
+    <br />"""
 
 
 def codestr2rst(codestr, lang='python', lineno=None):
-    """Return reStructuredText code block from code string"""
+    """Return reStructuredText code block from code string."""
     if lineno is not None:
         # Sphinx only starts numbering from the first non-empty line.
         blank_lines = codestr.count('\n', 0, -len(codestr.lstrip()))
         lineno = '   :lineno-start: {0}\n'.format(lineno + blank_lines)
     else:
         lineno = ''
-    code_directive = "\n.. code-block:: {0}\n{1}\n".format(lang, lineno)
+    code_directive = ".. code-block:: {0}\n{1}\n".format(lang, lineno)
     indented_block = indent(codestr, ' ' * 4)
     return code_directive + indented_block
 
@@ -240,10 +241,10 @@ def extract_intro_and_title(filename, docstring):
     return intro, title
 
 
-def md5sum_is_current(src_file):
+def md5sum_is_current(src_file, mode='b'):
     """Checks whether src_file has the same md5 hash as the one on disk"""
 
-    src_md5 = get_md5sum(src_file)
+    src_md5 = get_md5sum(src_file, mode)
 
     src_md5_file = src_file + '.md5'
     if os.path.exists(src_md5_file):
@@ -384,15 +385,47 @@ def generate_dir_rst(src_dir, target_dir, gallery_conf, seen_backrefs):
 
 
 def handle_exception(exc_info, src_file, script_vars, gallery_conf):
+    """Trim and format exception, maybe raise error, etc."""
     from .gen_gallery import _expected_failing_examples
     etype, exc, tb = exc_info
     stack = traceback.extract_tb(tb)
-    # Remove our code from traceback:
-    if isinstance(exc, SyntaxError):
-        # Remove one extra level through ast.parse.
-        stack = stack[2:]
-    else:
-        stack = stack[1:]
+    # The full traceback will look something like:
+    #
+    #   File "/home/larsoner/python/sphinx-gallery/sphinx_gallery/gen_rst.py...
+    #     mem_max, _ = gallery_conf['call_memory'](
+    #   File "/home/larsoner/python/sphinx-gallery/sphinx_gallery/gen_galler...
+    #     mem, out = memory_usage(func, max_usage=True, retval=True,
+    #   File "/home/larsoner/.local/lib/python3.8/site-packages/memory_profi...
+    #     returned = f(*args, **kw)
+    #   File "/home/larsoner/python/sphinx-gallery/sphinx_gallery/gen_rst.py...
+    #     exec(self.code, self.fake_main.__dict__)
+    #   File "/home/larsoner/python/sphinx-gallery/sphinx_gallery/tests/tiny...
+    #     raise RuntimeError('some error')
+    # RuntimeError: some error
+    #
+    # But we should trim these to just the relevant trace at the user level,
+    # so we inspect the traceback to find the start and stop points.
+    start = 0
+    stop = len(stack)
+    root = os.path.dirname(__file__) + os.sep
+    for ii, s in enumerate(stack, 1):
+        # Trim our internal stack
+        if s.filename.startswith(root + 'gen_gallery.py') and \
+                s.name == 'call_memory':
+            start = max(ii, start)
+        elif s.filename.startswith(root + 'gen_rst.py'):
+            # SyntaxError
+            if s.name == 'execute_code_block' and ('compile(' in s.line or
+                                                   'save_figures' in s.line):
+                start = max(ii, start)
+            # Any other error
+            elif s.name == '__call__':
+                start = max(ii, start)
+            # Our internal input() check
+            elif s.name == '_check_input' and ii == len(stack):
+                stop = ii - 1
+    stack = stack[start:stop]
+
     formatted_exception = 'Traceback (most recent call last):\n' + ''.join(
         traceback.format_list(stack) +
         traceback.format_exception_only(etype, exc))
@@ -516,7 +549,7 @@ def execute_code_block(compiler, block, example_globals,
 
     sys_path = copy.deepcopy(sys.path)
     sys.path.append(os.getcwd())
-
+    need_save_figures = True
     try:
         dont_inherit = 1
         if sys.version_info >= (3, 8):
@@ -524,8 +557,8 @@ def execute_code_block(compiler, block, example_globals,
         else:
             ast_Module = ast.Module
         code_ast = ast_Module([bcontent])
-        code_ast = compile(bcontent, src_file, 'exec',
-                           ast.PyCF_ONLY_AST | compiler.flags, dont_inherit)
+        flags = ast.PyCF_ONLY_AST | compiler.flags
+        code_ast = compile(bcontent, src_file, 'exec', flags, dont_inherit)
         ast.increment_lineno(code_ast, lineno - 1)
         # capture output if last line is expression
         is_last_expr = False
@@ -555,6 +588,10 @@ def execute_code_block(compiler, block, example_globals,
                     compiler(code_ast, src_file, 'exec'),
                     script_vars['fake_main']))
         script_vars['memory_delta'].append(mem_max)
+        # This should be inside the try block, e.g., in case of a savefig error
+        logging_tee.restore_std()
+        need_save_figures = False
+        images_rst = save_figures(block, script_vars, gallery_conf)
     except Exception:
         logging_tee.restore_std()
         except_rst = handle_exception(sys.exc_info(), src_file, script_vars,
@@ -562,9 +599,9 @@ def execute_code_block(compiler, block, example_globals,
         code_output = u"\n{0}\n\n\n\n".format(except_rst)
         # still call this even though we won't use the images so that
         # figures are closed
-        save_figures(block, script_vars, gallery_conf)
+        if need_save_figures:
+            save_figures(block, script_vars, gallery_conf)
     else:
-        logging_tee.restore_std()
         sys.path = sys_path
         os.chdir(cwd)
 
@@ -599,10 +636,9 @@ def execute_code_block(compiler, block, example_globals,
             captured_std = CODE_OUTPUT.format(indent(captured_std, u' ' * 4))
         else:
             captured_std = ''
-        images_rst = save_figures(block, script_vars, gallery_conf)
         # give html output its own header
         if repr_meth == '_repr_html_':
-            captured_html = html_header.format(indent(last_repr, u' ' * 8))
+            captured_html = html_header.format(indent(last_repr, u' ' * 4))
         else:
             captured_html = ''
         code_output = u"\n{0}\n\n{1}\n{2}\n\n".format(
@@ -650,6 +686,11 @@ def executable_script(src_file, gallery_conf):
     return execute
 
 
+def _check_input(prompt=None):
+    raise ExtensionError(
+        'Cannot use input() builtin function in Sphinx-Gallery examples')
+
+
 def execute_script(script_blocks, script_vars, gallery_conf):
     """Execute and capture output from python script already in block structure
 
@@ -689,6 +730,8 @@ def execute_script(script_blocks, script_vars, gallery_conf):
         # want to print it
         '__doc__': '',
         # Don't ever support __file__: Issues #166 #212
+        # Don't let them use input()
+        'input': _check_input,
     })
     script_vars['example_globals'] = example_globals
 
@@ -698,7 +741,7 @@ def execute_script(script_blocks, script_vars, gallery_conf):
         # https://github.com/sphinx-gallery/sphinx-gallery/pull/252
         # for more details.
         sys.argv[0] = script_vars['src_file']
-        sys.argv[1:] = []
+        sys.argv[1:] = gallery_conf['reset_argv'](gallery_conf, script_vars)
         gc.collect()
         memory_start, _ = gallery_conf['call_memory'](lambda: None)
     else:
@@ -723,7 +766,7 @@ def execute_script(script_blocks, script_vars, gallery_conf):
         # Write md5 checksum if the example was meant to run (no-plot
         # shall not cache md5sum) and has built correctly
         with open(script_vars['target_file'] + '.md5', 'w') as file_checksum:
-            file_checksum.write(get_md5sum(script_vars['target_file']))
+            file_checksum.write(get_md5sum(script_vars['target_file'], 't'))
         gallery_conf['passing_examples'].append(script_vars['src_file'])
 
     return output_blocks, time_elapsed
@@ -756,7 +799,7 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf,
     seen_backrefs = set() if seen_backrefs is None else seen_backrefs
     src_file = os.path.normpath(os.path.join(src_dir, fname))
     target_file = os.path.join(target_dir, fname)
-    _replace_md5(src_file, target_file, 'copy')
+    _replace_md5(src_file, target_file, 'copy', mode='t')
 
     file_conf, script_blocks, node = split_code_and_text_blocks(
         src_file, return_node=True)
@@ -765,10 +808,15 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf,
 
     executable = executable_script(src_file, gallery_conf)
 
-    if md5sum_is_current(target_file):
+    if md5sum_is_current(target_file, mode='t'):
+        do_return = True
         if executable:
-            gallery_conf['stale_examples'].append(target_file)
-        return intro, title, (0, 0)
+            if gallery_conf['run_stale_examples']:
+                do_return = False
+            else:
+                gallery_conf['stale_examples'].append(target_file)
+        if do_return:
+            return intro, title, (0, 0)
 
     image_dir = os.path.join(target_dir, 'images')
     if not os.path.exists(image_dir):
@@ -808,10 +856,10 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf,
 
     save_thumbnail(image_path_template, src_file, file_conf, gallery_conf)
 
-    example_nb = jupyter_notebook(script_blocks, gallery_conf)
+    example_nb = jupyter_notebook(script_blocks, gallery_conf, target_dir)
     ipy_fname = replace_py_ipynb(target_file) + '.new'
     save_notebook(example_nb, ipy_fname)
-    _replace_md5(ipy_fname)
+    _replace_md5(ipy_fname, mode='t')
 
     # Write names
     if gallery_conf['inspect_global_variables']:
@@ -835,8 +883,34 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf,
     return intro, title, (time_elapsed, memory_used)
 
 
+EXAMPLE_HEADER = """
+.. DO NOT EDIT.
+.. THIS FILE WAS AUTOMATICALLY GENERATED BY SPHINX-GALLERY.
+.. TO MAKE CHANGES, EDIT THE SOURCE PYTHON FILE:
+.. "{0}"
+.. LINE NUMBERS ARE GIVEN BELOW.
+
+.. only:: html
+
+    .. note::
+        :class: sphx-glr-download-link-note
+
+        Click :ref:`here <sphx_glr_download_{1}>`
+        to download the full example code{2}
+
+.. rst-class:: sphx-glr-example-title
+
+.. _sphx_glr_{1}:
+
+"""
+RST_BLOCK_HEADER = """\
+.. GENERATED FROM PYTHON SOURCE LINES {0}-{1}
+
+"""
+
+
 def rst_blocks(script_blocks, output_blocks, file_conf, gallery_conf):
-    """Generates the rst string containing the script prose, code and output
+    """Generate the rst string containing the script prose, code and output.
 
     Parameters
     ----------
@@ -862,9 +936,14 @@ def rst_blocks(script_blocks, output_blocks, file_conf, gallery_conf):
     # A simple example has two blocks: one for the
     # example introduction/explanation and one for the code
     is_example_notebook_like = len(script_blocks) > 2
-    example_rst = u""  # there can be unicode content
-    for (blabel, bcontent, lineno), code_output in \
-            zip(script_blocks, output_blocks):
+    example_rst = ""
+    for bi, ((blabel, bcontent, lineno), code_output) in \
+            enumerate(zip(script_blocks, output_blocks)):
+        # do not add comment to the title block, otherwise the linking does
+        # not work properly
+        if bi > 0:
+            example_rst += RST_BLOCK_HEADER.format(
+                lineno, lineno + bcontent.count('\n'))
         if blabel == 'code':
 
             if not file_conf.get('line_numbers',
@@ -906,21 +985,15 @@ def save_rst_example(example_rst, example_file, time_elapsed,
         Sphinx-Gallery configuration dictionary
     """
 
-    ref_fname = os.path.relpath(example_file, gallery_conf['src_dir'])
-    ref_fname = ref_fname.replace(os.path.sep, "_")
+    example_fname = os.path.relpath(example_file, gallery_conf['src_dir'])
+    ref_fname = example_fname.replace(os.path.sep, "_")
 
     binder_conf = check_binder_conf(gallery_conf.get('binder'))
 
     binder_text = (" or to run this example in your browser via Binder"
                    if len(binder_conf) else "")
-    example_rst = (".. only:: html\n\n"
-                   "    .. note::\n"
-                   "        :class: sphx-glr-download-link-note\n\n"
-                   "        Click :ref:`here <sphx_glr_download_{0}>` "
-                   "    to download the full example code{1}\n"
-                   "    .. rst-class:: sphx-glr-example-title\n\n"
-                   "    .. _sphx_glr_{0}:\n\n"
-                   ).format(ref_fname, binder_text) + example_rst
+    example_rst = EXAMPLE_HEADER.format(
+        example_fname, ref_fname, binder_text) + example_rst
 
     if time_elapsed >= gallery_conf["min_reported_time"]:
         time_m, time_s = divmod(time_elapsed, 60)
@@ -945,6 +1018,10 @@ def save_rst_example(example_rst, example_file, time_elapsed,
     write_file_new = re.sub(r'\.py$', '.rst.new', example_file)
     with codecs.open(write_file_new, 'w', encoding="utf-8") as f:
         f.write(example_rst)
+    # make it read-only so that people don't try to edit it
+    mode = os.stat(write_file_new).st_mode
+    ro_mask = 0x777 ^ (stat.S_IWRITE | stat.S_IWGRP | stat.S_IWOTH)
+    os.chmod(write_file_new, mode & ro_mask)
     # in case it wasn't in our pattern, only replace the file if it's
     # still stale.
-    _replace_md5(write_file_new)
+    _replace_md5(write_file_new, mode='t')
